/** 
 * mapCanvas.java
 *
 * Description:	
 * @author			Nikita Mikros (c)2003 Black Hammer Game
 * @version			
 
 Draws the grid of the map and stores the data about the grid
 */
 
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import net.n3.nanoxml.*;                              

public class mapCanvas extends Canvas
{
	short mOffsetX;
	short mOffsetY;
	
	short 	mWidth = 0;
	short	mHeight = 0;
	int		mStartNode;
	final short mMaxWidth = 32;
	final short mMaxHeight = 32;
	
	public boolean mShowGrid = true;
	public boolean mShowTerrainType = true;;
	public boolean mShowStartPoints = true;
	public boolean mShowAltars = true;
	public boolean mShowProps = true;
	
	public int mGridTileSize = 8;
	public boolean mExportPaths = false;
	public boolean mShowBorder = true;
	
	public byte[][] mTiles = new byte[mMaxWidth][mMaxHeight];
	public int[][] mTileHeights = new int[mMaxWidth][mMaxHeight];
	public String[][]mProps = new String[mMaxWidth][mMaxHeight];
	public int[][] mRotations =new int[mMaxWidth][mMaxHeight];
	public int mObjectTypes[][] = new int[mMaxWidth][mMaxHeight];
	public int mObjectRace[][] = new int[mMaxWidth][mMaxHeight];
	public int mObjectOwner[][] = new int[mMaxWidth][mMaxHeight];
	public Dimension mPropSizes[][] = new Dimension[mMaxWidth][mMaxHeight];
	
	public short mNumProps;
	public short mNumObjects;
	
	//byte mTerrainHeightMap[mMaxWidth*mGridTileSize][mMaxHeight*mGridTileSize];
	
	public final byte OLD_TERRAIN_WATER = 1;
	public final byte OLD_TERRAIN_SPACE = 2;
	public final byte OLD_TERRAIN_NORMAL = 3;	//grass, paths, etc.
	public final byte OLD_TERRAIN_SLOW = 4;		//e.g. low hill
	public final byte OLD_TERRAIN_SLOWER = 5;	//e.g. halfway up a mountain
	public final byte OLD_TERRAIN_NOMOVE = 6;	//e.g. high mountain, deep water, etc.
	public final byte OLD_TERRAIN_FIREDMG = 7;	//e.g. lava
	public final byte OLD_TERRAIN_SWAMPDMG = 8;//e.g. swamp gas area
	//public final byte NUM_TERRAIN_TYPES = 8;


	public final byte TERRAIN_NONE = 0;
	public final byte TERRAIN_NOMOVE = 1;			// 1
	public final byte TERRAIN_SPACE = 2;			// 2
	public final byte TERRAIN_NORMAL = 3;			// 3 e.g. grass, paths, etc.
	public final byte TERRAIN_SLOW_ALTAN = 4;		// 4 e.g. grass
	public final byte TERRAIN_SLOW_GROTH = 5;		// 5 e.g. swamp
	public final byte TERRAIN_SLOW_CHAAR = 6;		// 6 e.g. rocks
	public final byte TERRAIN_SLOW_FVOST = 7;		// 7 e.g. icicles?
	public final byte NUM_TERRAIN_TYPES = 7;
	
	public final int EXPORT_NORMAL = 0;
	public final int EXPORT_TEXTURE = 1;
	public final int EXPORT_HEIGHT = 2;

	public final int PROP = 48;				
	public final int LIGHT_ALTAR = 49;
	public final int SWAMP_ALTAR = 50;
	public final int FIRE_ALTAR = 51;
	public final int ICE_ALTAR = 52;
	
	public final int LIGHT = 0;
	public final int SWAMP = 1;
	public final int FIRE = 2;
	public final int ICE = 3;
	
	int mCurrentPlanetType = LIGHT;
	
	short mAttackStartX = -1;
	short mAttackStartY = -1;
	short mDefendStartX = -1;
	short mDefendStartY = -1;
	
	short mMaxTroopsAggressor = 25;
	short mMaxTroopsDefender = 25;
	Image mGrassImage;
	Image mExportImage;

	FileOutputStream flog;// = new FileOutputStream("mapHelper.log");
	PrintStream mLog;// = new PrintStream(flog);
	
	String  mTerrainTypeStrings[]=
	{
		"NO MOVE",
		"SPACE",
		"NORMAL",
		"SLOW ALTAN",
		"SLOW GROTH",
		"SLOW CHAAR",
		"SLOW FVOST",
	};
	Color mTerrainColors[]=
	{
		new Color(0,0,255,96),//water
		new Color(0,0,0),//space
		new Color(128,128,128,0),//normal
		new Color(139,180,143,96),//slow
		new Color(69,179,80,96),//slower
		new Color(248,237,55,96),//no move
		new Color(255,0,0,96),//firedmg
		new Color(248,98,236,96)//swampdmg
	};
	Image mPropImages[] = new Image[4];
	Image mTerrainImages[] = new Image[NUM_TERRAIN_TYPES];
	Image mObjectImages[] = new Image[4];
	Image mTerrainTextureImages[] = new Image[NUM_TERRAIN_TYPES];
	Image mTerrainHeightImages[] = new Image[3];
	int mHeightMap[][] = new int[mMaxWidth*16][mMaxHeight*16];
	class Node
	{
		public short value;
		public short cost;
	}
	public boolean save(DataOutputStream out) throws Exception
	{
		//3 bytes magic
		out.write('m');
		out.write('a');
		out.write('p');
		
		
		//1 byte version
		out.write(7);

		//write out flag if paths are included or not
		out.write(mExportPaths?1:0);
		out.write(0);//reserved -- this will be the type of planet
		out.write(0);//reserved 
		out.write(0);//reserved
		
		//4 bytes size
		out.writeShort(bhgUtils.swapBytes(mWidth));
		out.writeShort(bhgUtils.swapBytes(mHeight));
		
		//4 bytes offset to map geometry
		out.writeShort(bhgUtils.swapBytes(mOffsetX));
		out.writeShort(bhgUtils.swapBytes(mOffsetY));
		System.out.println("offsets saved x y" +mOffsetX+" "+mOffsetY);
		
		//2 bytes max troops per side
		out.writeShort(bhgUtils.swapBytes(mMaxTroopsAggressor));
		out.writeShort(bhgUtils.swapBytes(mMaxTroopsDefender));
		//4 bytes start attack position
		short attackX = (short)(mAttackStartX - mOffsetX);
		short attackY = (short)(mAttackStartY - mOffsetY);
		short defendX = (short)(mDefendStartX - mOffsetX);
		short defendY = (short)(mDefendStartY - mOffsetY);
		
		out.writeShort(bhgUtils.swapBytes(attackX));
		out.writeShort(bhgUtils.swapBytes(attackY));
		
		//4 bytes start defend position
		out.writeShort(bhgUtils.swapBytes(defendX));
		out.writeShort(bhgUtils.swapBytes(defendY));		
		
		//write the tiles out 
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				out.write(mTiles[i][j]);
			}
		}
		//System.out.println("beginHeights");
		//write out the heights
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				byte b = (byte)mTileHeights[i][j];
				out.write(b);
				if(b < 16)
					System.out.print("0");
				System.out.print(Integer.toHexString(b));
			}
			System.out.println();
		}
		//write out props //version 4 and above
		out.writeShort(bhgUtils.swapBytes(mNumProps));
		if(mNumProps > 0)
		{
			for(short j = 0; j < mMaxHeight; j++)
			{
				for(short i = 0; i < mMaxWidth; i++)
				{
					if(mProps[i][j] != null && !mProps[i][j].equals(""))
					{
						out.writeShort(bhgUtils.swapBytes((short)(i)));	//x pos
						out.writeShort(bhgUtils.swapBytes((short)(j)));	//y pos
						out.writeShort(bhgUtils.swapBytes((short)(mRotations[i][j])));	//rotation
						byte[] tempString = bhgUtils.toBytes(mProps[i][j],256);
						out.write(tempString);
					}	
				}
			}
		
		}
		//write out objects //version 5 and above
		out.writeShort(bhgUtils.swapBytes(mNumObjects));
		if(mNumObjects > 0)
		{
			for(short j = 0; j < mMaxHeight; j++)
			{
				for(short i = 0; i < mMaxWidth; i++)
				{
					if(mObjectTypes[i][j] != 0)
					{
						short type = (short)mObjectTypes[i][j];
						short race = (short)mObjectRace[i][j];
						short owner = (short)mObjectOwner[i][j];
						
						out.writeShort(bhgUtils.swapBytes((short)(i)));
						out.writeShort(bhgUtils.swapBytes((short)(j)));
						out.writeShort(bhgUtils.swapBytes((short)(type)));
						out.writeShort(bhgUtils.swapBytes((short)(race)));
						out.writeShort(bhgUtils.swapBytes((short)(owner)));
					}	
				}
			}
		
		}
		
		if(mExportPaths)
		{
			//write out the solution table
			createSolutionTable();	//make sure one exists and is up to date
			for(int j = 0; j < mWidth*mHeight; j++)
				for(int i = 0; i < mWidth*mHeight; i++)
				{
					out.writeShort(bhgUtils.swapBytes(mSolutionTable[i][j].value));
				}	
			for(int j = 0; j < mWidth*mHeight; j++)
				for(int i = 0; i < mWidth*mHeight; i++)
				{
					out.writeShort(bhgUtils.swapBytes(mSolutionTable[i][j].cost));
				}		
		}
		//mSolutionTable
		return true;
	}
	public boolean read(DataInputStream in) throws Exception
	{
		byte version;
		byte[] magic = new byte[3];
		in.read(magic,0,3);

		if(magic[0] != 'm' || magic[1] != 'a' || magic[2] != 'p')
			return false;
		version = (byte)in.read();
		if(version >1)
		{
			mExportPaths = (in.read() == 1);
			for(int i = 0; i < 3; i++)
				in.read();
		}
		if(version < 8)
		{
			//4 bytes size
			mWidth = bhgUtils.swapBytes(in.readShort());
			mHeight = bhgUtils.swapBytes(in.readShort());
		
			//4 bytes offset
			mOffsetX = bhgUtils.swapBytes(in.readShort());
			mOffsetY = bhgUtils.swapBytes(in.readShort());
			
			if(version == 1)
			{
				//2 bytes max troops per side
				mMaxTroopsAggressor = mMaxTroopsDefender = bhgUtils.swapBytes(in.readShort());
			}
			else if(version >= 2)
			{
				//2 bytes max troops per side
				mMaxTroopsAggressor = bhgUtils.swapBytes(in.readShort());
				mMaxTroopsDefender = bhgUtils.swapBytes(in.readShort());
			}
			//4 bytes start attack position
			mAttackStartX = bhgUtils.swapBytes(in.readShort());
			mAttackStartY = bhgUtils.swapBytes(in.readShort());
			//4 bytes start defend position
			mDefendStartX = bhgUtils.swapBytes(in.readShort());
			mDefendStartY = bhgUtils.swapBytes(in.readShort());	
			
			//adjust attack and defend positions
			//if(version < 3)
			{
				mAttackStartX+=mOffsetX;
				mAttackStartY+=mOffsetY;
				mDefendStartX+=mOffsetX;
				mDefendStartY+=mOffsetY;
			}
			
			if(version < 3)
			{					
				for(int j = mOffsetY; j < mOffsetY+mHeight; j++)
				{
					for(int i = mOffsetX; i < mOffsetX+mWidth; i++)
					{
						mTiles[i][j] = (byte)in.read();
					}
				}
			}
			else
			{
				for(int j = 0; j < mMaxHeight; j++)
				{
					for(int i = 0; i < mMaxWidth; i++)
					{
						byte tile = (byte)in.read();
						if(version < 5)
						{
							switch(tile)
							{
								case OLD_TERRAIN_NOMOVE:
									tile = TERRAIN_NOMOVE;
									break;
								case OLD_TERRAIN_SLOW:
								case OLD_TERRAIN_SLOWER:
								case OLD_TERRAIN_FIREDMG:
								case OLD_TERRAIN_SWAMPDMG:
								case OLD_TERRAIN_WATER:
									tile = TERRAIN_NORMAL;
									break;
								default:
									break;
							}
						
						}
						
						mTiles[i][j] = tile;
					}
				}
				for(int j = 0; j < mMaxHeight; j++)
				{
					for(int i = 0; i < mMaxWidth; i++)
					{
						byte height = (byte)in.read();
						mTileHeights[i][j] = (int)height;
					}
				}
				
			}
			if(version >= 4)
			{
				mNumProps = bhgUtils.swapBytes(in.readShort());	
				for(int i = 0; i < mNumProps; i++)
				{
					int x = bhgUtils.swapBytes(in.readShort());	
					int y = bhgUtils.swapBytes(in.readShort());	
					if(version >= 7)
						mRotations[x][y] = bhgUtils.swapBytes(in.readShort());
					else
						mRotations[x][y] = 0;
					byte[] tempBytes = new byte[256];
					in.read(tempBytes,0,256);
					String s = new String(tempBytes);
					mProps[x][y] = s.trim();
				
				}
			
			}
			if(version >=5)
			{
				mNumObjects = bhgUtils.swapBytes(in.readShort());
				for(int i = 0; i < mNumObjects; i++)
				{
					int x = bhgUtils.swapBytes(in.readShort());	
					int y = bhgUtils.swapBytes(in.readShort());	
					
					mObjectTypes[x][y] = bhgUtils.swapBytes(in.readShort());
					if(version < 6)
						mObjectTypes[x][y]++;				
					mObjectRace[x][y] = bhgUtils.swapBytes(in.readShort());				
					mObjectOwner[x][y] = bhgUtils.swapBytes(in.readShort());				
				}
			}
			repaint();
			return true;
		}
		return false;
	}
	public void exportBMP(DataOutputStream out, int type) throws Exception
	{
		int tile_size = type==EXPORT_NORMAL?8:16;
		//bit o magic
		out.writeByte('B');
		out.writeByte('M');
		
		//complet file size
		out.writeInt(bhgUtils.swapBytes((int)(0x36 + mMaxWidth*mMaxHeight*64*4)));
		
		//reserved
		out.writeInt(0);
		
		//offset to the beginning of the bitmap data 
		out.writeInt(bhgUtils.swapBytes((int)0x36));
		
		//header size
		out.writeInt(bhgUtils.swapBytes((int)0x28));
		//width
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*tile_size));
		//height
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*tile_size));
		
		//planes
		out.writeShort(bhgUtils.swapBytes((short)1));
		//bits per pixel
		out.writeShort(bhgUtils.swapBytes((short)32));
		
		//compression (none)
		out.writeInt(bhgUtils.swapBytes((int)0));
		
		//size of the bitmap (in bytes)
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*mMaxWidth *tile_size*tile_size*4));
		
		//hor  & vert rez
		out.writeInt(0);
		out.writeInt(0);
		
		//colors
		out.writeInt(0);
		//important colors
		out.writeInt(0);
		
		int x,y;
		
		int[] pixels = new int[mMaxWidth * mMaxHeight*tile_size*tile_size];
		int pixWidth = mMaxWidth*tile_size;
		int pixHeight = mMaxHeight*tile_size;
		//Image im =  createImage(pixWidth, pixHeight);
		if(mExportImage == null)
		{
			mExportImage =  createImage(mMaxWidth*tile_size, mMaxHeight*tile_size);

		}
			//this.primeImages();
		Image im = mExportImage;
		Graphics ig = im.getGraphics();

		if(type == EXPORT_TEXTURE)
			paintTerrain(ig);
		else if(type == EXPORT_HEIGHT)
			paintHeightMap(ig);
		else
			paint(ig);
		
		
		PixelGrabber pg = new PixelGrabber(im, 0, 0, pixWidth, pixHeight, pixels, 0, pixWidth);
		try 
		{
	    	pg.grabPixels();
		} 
		catch (InterruptedException e) 
		{
	    	System.err.println("interrupted waiting for pixels!");
	    }

		
		for(int j = 0; j < pixHeight; j++)
		{
			for(int i = 0; i < pixWidth;i++)
			{
				x = i;
				y = pixHeight - (j+1);
				/*if	(y >= (mOffsetY * 8) && y < ((mOffsetY + mHeight) * 8) && 
					(x >= (mOffsetX * 8) && x < ((mOffsetX + mWidth) * 8)))
					{*/
						int pixX = i;// - (mOffsetX*8);
						int pixY = y;//- (mOffsetY*8);
						
						//System.out.println("pixX"+pixX+"pixY"+pixY+"pixWidth"+pixWidth);
						//System.out.println("j"+j+"mOffsetY*8"+mOffsetY*8+"y"+y);
						out.writeInt(bhgUtils.swapBytes(pixels[pixY * pixWidth + pixX]));
					
				/*	}
					
				else
					out.writeInt(0xffffff00);*/
			}
		}
		
	
	}
	//Image mDrawingImage = null;
/*	public boolean init(DataInputStream in) throws Exception
	{
		int w=0,h=0;
		int bitsPerPixel = 0;
		
		//establish that this is actually a bmp file
		String s = "";
		s += (char)in.readByte();
		s += (char)in.readByte();
		if (!s.equals("BM"))
			throw new Exception();
		System.out.println(""+bhgUtils.swapBytes(in.readInt())); 
		System.out.println(""+bhgUtils.swapBytes(in.readShort())); 
		System.out.println(""+bhgUtils.swapBytes(in.readShort())); 
		System.out.println(""+bhgUtils.swapBytes(in.readInt())); 
		
		//now read the bitmap info header
		System.out.println("size of header "+bhgUtils.swapBytes(in.readInt())); 
		w = bhgUtils.swapBytes(in.readInt());
		h = bhgUtils.swapBytes(in.readInt());
		System.out.println("w "+w); 
		System.out.println("h "+h); 
		System.out.println("planes "+bhgUtils.swapBytes(in.readShort())); 
		
		bitsPerPixel = bhgUtils.swapBytes(in.readShort());
		System.out.println("bits per pixel "+bitsPerPixel); 
		if(bitsPerPixel != 8)
		{
			throw new Exception();
		}
		System.out.println("compression (should be 0)"+bhgUtils.swapBytes(in.readInt())); 
		System.out.println("size of the image"+bhgUtils.swapBytes(in.readInt())); 
		System.out.println(""+bhgUtils.swapBytes(in.readInt())); 
		System.out.println(""+bhgUtils.swapBytes(in.readInt())); 
		int numColors = bhgUtils.swapBytes(in.readInt());
		if(numColors == 0)
			numColors = 256;
		System.out.println("num colors"+numColors); 
		System.out.println("num important colors"+bhgUtils.swapBytes(in.readInt())); 
		byte[] colors = new byte[256 * 4];
		colors[0] = 0;
		colors[1] = 0;
		colors[2] = 0;
		colors[3] = 0;
		for(int j = 1; j < numColors; j++)
		{
			colors[j*4 + 2] = in.readByte();//blue
			colors[j*4 + 1] = in.readByte();//green
			colors[j*4 + 0] = in.readByte();//red
			in.readByte();//ignore this
			colors[j*4 + 3] =(byte)(0xFF);
		}

		
			//System.out.println(""+(in.readUnsignedByte())+" "+in.readUnsignedByte()+" "+in.readUnsignedByte()+" "+in.readUnsignedByte());
		mColorModel = new IndexColorModel(
													8, 
													256,
													colors, 
													0,
													true
													,0);
		setSize(w,h);
		//mImage = new BufferedImage(w,h,BufferedImage.TYPE_BYTE_INDEXED,cm);
		mImageData = new byte[w*h];
		//in.skip(1070-308);
		
		
        //Raster tempRaster = mImage.getData();     
        //DataBuffer tempData = tempRaster.getDataBuffer();  
        for(int j = h-1; j >= 0; j--)
        {
        	in.readFully(mImageData,j*w,w);	      
	    }
        //mImage.setData(tempRaster);
        //if(mImage.getType()!= BufferedImage.TYPE_BYTE_INDEXED)
        //	throw new Exception();
		
		mImage = createImage(new MemoryImageSource(w, h, mColorModel, mImageData, 0, w));
        mWidth = w;
        mHeight = h;
        //repaint();
        return true;

	}*/
	
	//constructor
	public mapCanvas()
	{
		clear(TERRAIN_NORMAL);
		clearHeights(8);
		clearProps();
		clearObjects();
		Toolkit t = java.awt.Toolkit.getDefaultToolkit();
//		for(int i = 0; i < NUM_TERRAIN_TYPES; i++)

		mTerrainImages[0] = t.getImage("stop.gif");
		mTerrainImages[1] = t.getImage("nothing.gif");
		mTerrainImages[2] = t.getImage("nothing.gif");
		mTerrainImages[3] = t.getImage("slow_altan.gif");
		mTerrainImages[4] = t.getImage("slow_groth.gif");
		mTerrainImages[5] = t.getImage("slow_chaar.gif");
		mTerrainImages[6] = t.getImage("slow_fvost.gif");
		
		mPropImages[0] = t.getImage("prop0.gif");
		mPropImages[1] = t.getImage("prop90.gif");
		mPropImages[2] = t.getImage("prop180.gif");
		mPropImages[3] = t.getImage("prop270.gif");
		for(int i = 0; i < 4; i++)
			t.prepareImage(mPropImages[i],-1,-1,null);
		for(int i = 0; i < NUM_TERRAIN_TYPES; i++)
			t.prepareImage(mTerrainImages[i],-1,-1,null);
			
		mTerrainTextureImages[TERRAIN_SLOW_ALTAN-1] = t.getImage("terrain/slow_altan.gif");
		t.prepareImage(mTerrainTextureImages[TERRAIN_SLOW_ALTAN-1],-1,-1,null);
		mTerrainTextureImages[TERRAIN_SLOW_GROTH-1] = t.getImage("terrain/slow_groth.gif");
		t.prepareImage(mTerrainTextureImages[TERRAIN_SLOW_GROTH-1],-1,-1,null);
		mTerrainTextureImages[TERRAIN_SLOW_CHAAR-1] = t.getImage("terrain/slow_chaar.gif");
		t.prepareImage(mTerrainTextureImages[TERRAIN_SLOW_CHAAR-1],-1,-1,null);
		mTerrainTextureImages[TERRAIN_SLOW_FVOST-1] = t.getImage("terrain/slow_fvost.gif");
		t.prepareImage(mTerrainTextureImages[TERRAIN_SLOW_FVOST-1],-1,-1,null);


		mObjectImages[0] = t.getImage("altar_altan.gif");
		t.prepareImage(mObjectImages[0],-1,-1,null);
		mObjectImages[1] = t.getImage("altar_groth.gif");
		t.prepareImage(mObjectImages[1],-1,-1,null);
		mObjectImages[2] = t.getImage("altar_chaar.gif");
		t.prepareImage(mObjectImages[2],-1,-1,null);
		mObjectImages[3] = t.getImage("altar_fvost.gif");
		t.prepareImage(mObjectImages[3],-1,-1,null);
		
		mTerrainHeightImages[0] = t.getImage("terrain/altan/low.gif");	
		mTerrainHeightImages[1] = t.getImage("terrain/altan/middle.gif");		
		mTerrainHeightImages[2] = t.getImage("terrain/altan/high.gif");		
		try
		{
			flog = new FileOutputStream("mapHelper.log");
			mLog = new PrintStream(flog);
		}catch(Exception e)
		{
			e.printStackTrace();
		}

	}
	public void changeTerrainImages(String race)
	{
		for(int i = 0; i < 3; i++)
		{
			mTerrainHeightImages[i].flush();
			mTerrainHeightImages[i] = null;
		}
		Toolkit t = java.awt.Toolkit.getDefaultToolkit();
		mTerrainHeightImages[0] = t.getImage("terrain/"+race+"/low.gif");	
		mTerrainHeightImages[1] = t.getImage("terrain/"+race+"/middle.gif");		
		mTerrainHeightImages[2] = t.getImage("terrain/"+race+"/high.gif");		
	}
	public void clear(byte clearColor)
	{
		//zero out the map
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				mTiles[i][j] = clearColor;
			}
		}
	}
	public void clearProps()
	{
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				mProps[i][j] = null;
				mPropSizes[i][j] = null;
			}
		}
	}
	public void clearHeights(int value)
	{
		//zero out the map
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				mTileHeights[i][j] = value;
			}
		}
	
	}
	public void clearObjects()
	{
		//zero out the map
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				mObjectTypes[i][j] = 0;
				mObjectRace[i][j] = 0;
				mObjectOwner[i][j] = 0;
			}
		}
	}
	public void update(Graphics g)
	{
		paint(g);
	}
	public void setProp(int x, int y, String propName, int rotation)
	{
		if(propName.equals("") || propName == null)
		{
			if(mProps[x][y] != null)
			{
				mProps[x][y] = null;
				mPropSizes[x][y] =null;
				mNumProps--;
			}
		}
		else
		{
			if(mProps[x][y] == null)
				mNumProps++;
			mProps[x][y] = propName;
			mPropSizes[x][y] = getPropSize(propName);
			mRotations[x][y] = rotation;
			mTiles[x][y] = TERRAIN_NOMOVE;
			if(mPropSizes[x][y].width > 1 || mPropSizes[x][y].height > 1)
			{
				int w = mPropSizes[x][y].width;
				int h = mPropSizes[x][y].height;
				
				if(rotation == 90 || rotation == 270)
				{
					int temp = w;
					w = h;
					h = temp;
				
				}
				int tx = x - w/2;
				int ty = y - h/2;
				
				for(int j = 0; j < h; j++)
				{
					for(int i = 0; i < w; i++)
					{	
						//if((tx+i) != x || (ty+j) != y)
						mTiles[tx+i][ty+j] = TERRAIN_NOMOVE;
					}
				}
			
			}
		}
		paint(getGraphics());
	}
	public Dimension getPropSize(String propName)
	{
		Dimension d = new Dimension(1,1);
		try
		{
			IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
			IXMLReader reader = StdXMLReader.fileReader("../../media/props/"+propName);
			parser.setReader(reader);
			IXMLElement xml = (IXMLElement) parser.parse();
			java.util.Vector v = xml.getChildrenNamed("size");
			if(v.size() > 0)
			{
				IXMLElement size = (IXMLElement)v.firstElement();
				d.width = Integer.parseInt(size.getAttribute("width"));
				d.height = Integer.parseInt(size.getAttribute("height"));
			}
			System.out.println("height ="+d.height+" width= "+d.width);
			return d;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return d;
		}
	}
	public void setObject(int x, int y, int type, int race, int owner)
	{
		if(	 
			x >= mOffsetX && x < mOffsetX+mWidth &&
			y >= mOffsetY && y < mOffsetY+mHeight)
		{
			if(type == 0 && mObjectTypes[x][y] != 0)
				mNumObjects--;
			else if(type != 0 && mObjectTypes[x][y] == 0)
				mNumObjects++;
			
			mObjectTypes[x][y] = type;
			mObjectRace[x][y] = race;
			mObjectOwner[x][y] = owner;
			paintArea(getGraphics(),x,y, 1, 1);
		}

	}
	public void setTile(int x, int y, byte value)
	{
		if(	value != mTiles[x][y] && 
			x >= mOffsetX && x < mOffsetX+mWidth &&
			y >= mOffsetY && y < mOffsetY+mHeight)
		{
			//first set the value
			mTiles[x][y] = value;
			
			//now draw it
			paintTile(getGraphics(),x,y);
			paintAttackStartPoint(getGraphics());
			paintDefendStartPoint(getGraphics());
			/*
			Graphics g = getGraphics();
			g.setColor(mTerrainColors[mTiles[x][y] - 1]);
			g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);	
			g.setColor(Color.lightGray);
			g.drawRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
			*/
		}
	}
	public void setHeight(int origX, int origY, int origHeight, int spread)
	{
		
		float falloff = (float)(origHeight)/(float)spread;
		
		for(int j = -spread; j <= spread; j++)
		{
			for(int i = -spread; i <= spread; i++)
			{
				int x = origX + i;
				int y = origY + j;
				int multiplier = Math.abs(j) + Math.abs(i);
				float height = origHeight;
				if(	 
					x >= 0 && x < mMaxWidth &&
					y >= 0 && y < mMaxHeight)
				{
					for(int k = multiplier; k > 0; k--)
					{
						/*height -= falloff;
						if(Math.abs(height) < .1)
						{
							break;
						}*/
						height/=2.0;
					} 

					//first set the value
					mTileHeights[x][y] += (int)(height);
					if(mTileHeights[x][y] < 0)
						mTileHeights[x][y] = 0;
					if(mTileHeights[x][y] > 15)
						mTileHeights[x][y] = 15;
					//now draw it
					//paintTile(getGraphics(),x,y);
					//paintAttackStartPoint(getGraphics());
					//paintDefendStartPoint(getGraphics());
					/*
					Graphics g = getGraphics();
					g.setColor(mTerrainColors[mTiles[x][y] - 1]);
					g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);	
					g.setColor(Color.lightGray);
					g.drawRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
					*/
				}
			}
		}
	}
	public void setZoom(int size)//1x 2x etc.
	{
		mGridTileSize = 8 * size;
		repaint();
	}
	public void setMapSize(short width, short height)
	{
		mWidth = width;
		mHeight = height;	
		repaint();
	}
	public void setStartPoint(int x, int y, int value)
	{
		if(x < mOffsetX || x >= (mOffsetX + mWidth) || y < mOffsetY || y >= (mOffsetY+mHeight))
			return;
			
		switch(value)
		{
			case 1: 
				if(x == mDefendStartX && y == mDefendStartY)
					return;
				if(mAttackStartX >= 0)
					paintTile(getGraphics(),mAttackStartX,mAttackStartY);
				mAttackStartX =(short) x;
				mAttackStartY =(short) y;
				paintAttackStartPoint(getGraphics());
				break;
			case 2:
				if(x == mAttackStartX && y == mAttackStartY)
					return;
				if(mDefendStartX >= 0)
					paintTile(getGraphics(),mDefendStartX,mDefendStartY);
				mDefendStartX = (short)x;
				mDefendStartY = (short)y;
				paintDefendStartPoint(getGraphics());
				break;
		}
	}
	public boolean isDoubleBuffered()
	{
		return true;
	}
	public int paintPath(int startX, int startY, int destX, int destY)
	{
		int start = getNodeFromXY(startX,startY);
		int dest = getNodeFromXY(destX,destY);
		int cost = mSolutionTable[dest][start].cost;
		int x,y;

		Graphics g = getGraphics();
		
		//draw start
		g.setColor(Color.green);
		x = getXfromNode(start);
		y = getYfromNode(start);
		g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);	

		g.setColor(Color.yellow);
		do
		{
			start = mSolutionTable[dest][start].value;
			x = getXfromNode(start);
			y = getYfromNode(start);
			if(start == dest)
				g.setColor(Color.red);
			g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);	
		}while(start != dest);
		return cost;
	}
	public void createHeightMap()
	{
		
		int tile_size = 16;
		int num_heights = 16;
		int double_tile_size = tile_size*2;
		
		int pixWidth = tile_size*mMaxWidth;
		int pixHeight = tile_size*mMaxHeight;
		int temp[][] = new int[pixWidth][mMaxHeight];
		
		//do the first pass horizontally
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < pixWidth; i++)
			{
				int x = i/tile_size;
				int left = mTileHeights[x-1>=0?x-1:x][j]*num_heights;
				int right = mTileHeights[x+1<mMaxWidth?x+1:x][j]*num_heights;
				int center = mTileHeights[x][j]*num_heights;
				
				int remainder = i%tile_size;
				remainder -= (tile_size/2);
				float influence = Math.abs((float)remainder/tile_size);
				int height;

				height = (int)(influence*(remainder<0?left:right) + (1.0 - influence)*center);
				//mLog.println("height before clipping for "+i+","+j+" = "+height);
				if(height>255)
					height = 255;
				if(height<0)
					height = 0;
					
				temp[i][j] = height;					
				
			}
		}
		//now the vertical
		for(int j = 0; j < pixHeight; j++)
		{
			for(int i = 0; i < pixWidth; i++)
			{
				int y = j/tile_size;
				int up = temp[i][y-1>=0?y-1:y];
				int down = temp[i][y+1<mMaxHeight?y+1:y];
				int center = temp[i][y];
				
				int remainder = j%tile_size;
				remainder -= (tile_size/2);
				float influence = Math.abs((float)remainder/tile_size);
				int height;

				height = (int)(influence*(remainder<0?up:down) + (1.0 - influence)*center);
				if(height>255)
					height = 255;
				if(height<0)
					height = 0;
					
				mHeightMap[i][j] = height;
				
				
			}
		}
	}
	public void paintHeightMap(Graphics g)
	{
		int tile_size = 16;
		createHeightMap();
		int pixWidth = mMaxWidth*tile_size;
		int pixHeight = mMaxHeight*tile_size;
		
		int temp[] = new int[pixWidth*pixHeight];
 		for(int j = 0; j < pixHeight; j++)
 			for(int i = 0; i < pixWidth; i++)
 			{
 				int c = mHeightMap[i][j];
 				temp[j*pixWidth+i] = 255<<24| c<<16|c<<8|c;
 			}
 		Image img = createImage(new MemoryImageSource(pixWidth, pixHeight, temp, 0, pixWidth));
 		g.drawImage(img,0,0,null);
 		img.flush();
	
	}
	public void paintTerrain(Graphics graf)
	{
		createHeightMap();
		int tile_size = 16;
		graf.setColor(Color.black);
		graf.fillRect(0,0,mMaxWidth*tile_size,mMaxHeight*tile_size);
		graf.setColor(Color.darkGray);
		for(int j = 0; j <= mMaxHeight*tile_size; j+= tile_size)
			graf.drawLine(0,j,mMaxWidth*tile_size,j);
		for(int i = 0; i <= mMaxWidth*tile_size; i+= tile_size)
			graf.drawLine(i,0,i,mMaxHeight*tile_size);
		
		graf.setColor(Color.red);
		graf.fillRect(mOffsetX*tile_size,mOffsetY*tile_size,mWidth*tile_size,mHeight*tile_size);
		graf.drawImage(mTerrainHeightImages[0],
			mOffsetX*tile_size, mOffsetY*tile_size, 
			(mOffsetX+mWidth)*tile_size, (mOffsetY+mWidth)*tile_size,  
			mOffsetX*tile_size, mOffsetY*tile_size, 
			(mOffsetX+mWidth)*tile_size, (mOffsetY+mWidth)*tile_size,  
			this);
		int pixWidth = mWidth*tile_size;
		int pixHeight = mHeight*tile_size;
		int pixX = mOffsetX*tile_size;
		int pixY = mOffsetY*tile_size;
		
		int heights[][] = new int[3][pixWidth*pixHeight];
		int tempPixels[] = new int[pixWidth*pixHeight];
		for(int i = 0; i < 3; i++)
		{
			PixelGrabber pg = new PixelGrabber(mTerrainHeightImages[i], pixX, pixY, pixWidth, pixHeight, heights[i], 0, pixWidth);
			try 
			{
				pg.grabPixels();
			} 
			catch (InterruptedException e) 
			{
			    System.err.println("interrupted waiting for pixels!");
			    return;
			}
		}
		
		
		for(int j = 0; j < pixHeight; j++)
		{
			for(int i = 0; i < pixWidth; i++)
			{
				int value = mHeightMap[pixX+i][pixY+j];
				float shading = 0;
				
				for(int j1 = -4; j1 <=4; j1++)
				{
					for(int i1 = -4; i1 <= 4; i1++)
					{
						if(j+j1 < 0 || j+j1 >= pixHeight || i+i1 < 0 || i+i1 >= pixWidth)
							continue;
						if(j1 == 0 && i1 == 0)
							continue;
						float diff = Math.abs(mHeightMap[pixX+i+i1][pixY+j+j1]-value);
						diff/= (Math.abs(j1)+Math.abs(i1));
						shading += diff;
					
					}
				
				} 
				shading *= .4;
				value -= 128;
				
				float influence = (float)Math.abs((float)value/128.0);
				
				int r1 = (heights[value<0?0:2][i+j*pixWidth] & 0x00FF0000)>>16;
				int g1 = (heights[value<0?0:2][i+j*pixWidth] & 0x0000FF00)>>8;
				int b1 = heights[value<0?0:2][i+j*pixWidth] & 0x000000FF;
				
				int r = (heights[1][i+j*pixWidth] & 0x00FF0000)>>16;
				int g = (heights[1][i+j*pixWidth] & 0x0000FF00)>>8;
				int b = heights[1][i+j*pixWidth] & 0x000000FF;
				
				r = (int)((influence*r1)+((1.0 - influence)*r));
				g = (int)((influence*g1)+((1.0 - influence)*g));
				b = (int)((influence*b1)+((1.0 - influence)*b));
				
				
				r-=shading;
				g-=shading;
				b-=shading;
				
				if(r<0)
					r=0;
				if(r>255)
					r=255;
				if(g<0)
					g=0;
				if(g>255)
					g=255;
				if(b<0)
					b=0;
				if(b>255)
					b=255;
				tempPixels[i+j*pixWidth] = 255<<24|r<<16|g<<8|b;
			}
		}
 		Image img = createImage(new MemoryImageSource(pixWidth, pixHeight, tempPixels, 0, pixWidth));
		
		graf.drawImage(img,pixX,pixY,null);
		//g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER));
		for(int j = mOffsetY; j < mOffsetY+mHeight; j++)
		{
			for(int i = mOffsetX; i < mOffsetX+mWidth; i++)
			{
				if(mTiles[i][j] > 0)
				{
					
					if(mTiles[i][j] >= TERRAIN_SLOW_ALTAN && mTiles[i][j] <= TERRAIN_SLOW_FVOST)
						graf.drawImage(mTerrainTextureImages[mTiles[i][j]],
						i*tile_size, j*tile_size, 
						(i+1)*tile_size, (j+1)*tile_size,  
						i*tile_size, j*tile_size, 
						(i+1)*tile_size, (j+1)*tile_size,  
						this);
				}
				
			}
		}
		
		//draw the border
		graf.setColor(Color.red);
		graf.drawRect(mOffsetX*tile_size,mOffsetY*tile_size,mWidth*tile_size,mHeight*tile_size);
		img.flush();

	}
	public void paintTile(Graphics g, int x, int y)
	{
		paintArea(g,x,y,1,1);
	}
	public void paintAttackStartPoint(Graphics g)
	{
		if(mAttackStartX >= 0)
		{
			g.setColor(Color.black);
			int offset = mGridTileSize/4;
			int x = (mAttackStartX*mGridTileSize)+offset;
			int y = (mAttackStartY*mGridTileSize)+offset;
			g.fillRect(x, y,mGridTileSize - (offset*2), mGridTileSize - (offset*2));
			
			g.setColor(Color.red);
			offset += mGridTileSize/8;
			x = (mAttackStartX*mGridTileSize)+offset;
			y = (mAttackStartY*mGridTileSize)+offset;
			g.fillRect(x, y,mGridTileSize - (offset*2), mGridTileSize - (offset*2));
		}
	}
	public void paintDefendStartPoint(Graphics g)
	{
		if(mDefendStartX >= 0)
		{
			g.setColor(Color.black);
			int offset = mGridTileSize/4;
			int x = (mDefendStartX*mGridTileSize)+offset;
			int y = (mDefendStartY*mGridTileSize)+offset;
			g.fillRect(x, y,mGridTileSize - (offset*2), mGridTileSize - (offset*2));
			
			g.setColor(Color.green);
			offset += mGridTileSize/8;
			x = (mDefendStartX*mGridTileSize)+offset;
			y = (mDefendStartY*mGridTileSize)+offset;
			g.fillRect(x, y,mGridTileSize - (offset*2), mGridTileSize - (offset*2));
		}
	}
	public void paintArea(Graphics g, int x, int y, int w, int h)
	{
		if(x < 0)
			x = 0;
		if(y < 0)
			y=0;
		if(x >= mMaxWidth)
			x = mMaxWidth-1;
		if(y >= mMaxHeight)
			y = mMaxHeight -1;
			
		if ((x+w) > mMaxWidth)
			w = mMaxWidth-x;
		if((y+h) > mMaxHeight)
			h = mMaxHeight-y;
		//g.clearRect(x*mGridTileSize, y * mGridTileSize,w*mGridTileSize, h*mGridTileSize);
		
		for(int j = y; j < y+h; j++)
		{
			for(int i = x; i < x+w; i++)
			{
				int c = (mTileHeights[i][j]*16);
				g.setColor(new Color(c,c,c));
				g.fillRect(i*mGridTileSize,j*mGridTileSize,mGridTileSize,mGridTileSize);					
			}
		}
		if(mShowTerrainType)
		{	
			
			//g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER));
			for(int j = y; j < y+h; j++)
			{
				if(j < mOffsetY || j >= mOffsetY+mHeight)
					continue;//out of bounds
				for(int i = x; i < x+w; i++)
				{
					if(i < mOffsetX || i >= mOffsetX + mWidth)
						continue;//out of bounds
					if(mTiles[i][j] > 0)
					{
						if(mTiles[i][j] == TERRAIN_NORMAL || mTiles[i][j] == TERRAIN_SPACE)
							continue;
						
						g.drawImage(mTerrainImages[mTiles[i][j]-1],i*mGridTileSize,j*mGridTileSize,mGridTileSize,mGridTileSize,null);
					
					}
					
				}
			}
		}
		for(int j = 0; j < mMaxHeight; j++)
			for(int i = 0; i < mMaxWidth; i++)
			{
				if(mShowProps && mProps[i][j] != null)
				{
					int rotation = mRotations[i][j]/90;
					g.drawImage(mPropImages[rotation],i*mGridTileSize,j*mGridTileSize,mGridTileSize,mGridTileSize,null);
				}
				if(mShowAltars && mObjectTypes[i][j] != 0)
				{
					g.drawImage(mObjectImages[mObjectTypes[i][j] - LIGHT_ALTAR], i*mGridTileSize,j*mGridTileSize,mGridTileSize,mGridTileSize,null);
				}
			}
		
		g.setColor(Color.lightGray);
		if(mShowGrid)
		{
			for(int j = y*mGridTileSize; j <= (y+h)*mGridTileSize; j+= mGridTileSize)
				g.drawLine(x*mGridTileSize,j,(x+w)*mGridTileSize,j);
			for(int i = x*mGridTileSize; i <= (x+w)*mGridTileSize; i+= mGridTileSize)
				g.drawLine(i,y*mGridTileSize,i,(y+h)*mGridTileSize);
		}
		if(mShowStartPoints)
		{
			paintAttackStartPoint(g);
			paintDefendStartPoint(g);
		}			
		if(mShowBorder)
		{
			g.setColor(Color.red);
			g.drawRect(mOffsetX*mGridTileSize,mOffsetY*mGridTileSize,mWidth*mGridTileSize,mHeight*mGridTileSize);
		}
	
	}
	public void paint(Graphics g)
	{
		
		paintArea(g,0,0,mMaxWidth,mMaxHeight);	
	}	
	
	/*
			pathfinding stuff
	*/
	Node[][] mSolutionTable;
	int [] mDistances;
	int [] mPrevious;
	
	class shortestDistanceComparator implements Comparator
	{
        public int compare(Object Left, Object Right)
        {
            int left = ((Integer)Left).intValue();
            int right = ((Integer)Right).intValue();
            int shortestDistanceLeft = mDistances[left];
            int shortestDistanceRight = mDistances[right];
            
            if (shortestDistanceLeft > shortestDistanceRight)
            {
                return +1;
            }
            else if (shortestDistanceLeft < shortestDistanceRight)
            {
                return -1;
            }
            else // equal
            {   
/*            	if(left == right)//already in the set so don't put it in again
            		return 0;         
                
                int[] nodes= new int[4];	//left is even right is odd
                
                nodes[0] = Math.abs(getXfromNode(left) - getXfromNode(mStartNode));
                nodes[1] = Math.abs(getXfromNode(right) - getXfromNode(mStartNode));
                nodes[2] = Math.abs(getYfromNode(left) - getYfromNode(mStartNode));
                nodes[3] = Math.abs(getYfromNode(right) - getYfromNode(mStartNode));
                int max = -1;
                int rv = 0;
                int numMatches = 0;
                for(int i = 0; i < 4; i++)
                {
                	if(nodes[i] == max)
                		numMatches++;
                	if(nodes[i] > max)
                	{
                		max = nodes[i];
                		rv = nodes[i]%2==0?1:-1;
                		numMatches = 1;
                	}	
                }
                if(rv != 0 && numMatches == 1)
                	return rv;*/
                if(left> right)
                	return +1;
                else if (left < right)
                	return -1;
                else
                	return 0;
            }
        }
    };
    	
	void createSolutionTable()
	{
		mSolutionTable = new Node[mWidth*mHeight][mWidth*mHeight];
		
		//initialize the solution table to -2 (unresolved)
		for(int j = 0; j < mWidth*mHeight; j++)
		{
			for(int i = 0; i < mWidth*mHeight; i++)
			{
				mSolutionTable[i][j] = new Node();
				mSolutionTable[i][j].value = -1;
				mSolutionTable[i][j].cost = 0;
			}
		}
				
		//for each unresolved entry find the paths
		for(int si = 0; si < mWidth*mHeight; si++)
		{
			mStartNode = si;
			findPath();
			setStatus("all paths found for x:"+getXfromNode(si)+"y:"+getYfromNode(si));
		}
		setStatus("create cost table");
		for(int si = 0; si < mWidth*mHeight; si++)
		{
			createCostTableFrom(si);
			setStatus("cost calculated for "+si);
		}
		setStatus("solution table created");
		//printSolutionTable();
	}
	void setStatus(String s)
	{
		mhFrame theParent = (mhFrame)this.getParent();
		theParent.setStatus(s);
	
	}
	//backtrack each path and calculate the cost
	void createCostTableFrom(int start)
	{

		int numNodes = mWidth*mHeight;

		
		for(int dest = 0; dest < numNodes; dest++)
		{
			
			int i;
			i = start;
			if(start == dest)
			{
				mSolutionTable[dest][start].cost = 0;
				continue;
			}
			do
			{
				i = mSolutionTable[dest][i].value;
				int x = getXfromNode(i);
				int y = getYfromNode(i);
				int cost = getCost(mTiles[x][y]);
				mSolutionTable[dest][start].cost+=cost;
			}while(i != dest);
			
		}
	
	}
	void findPath()
	{
		int numNodes = mWidth*mHeight;
		mDistances = new int[numNodes];
		mPrevious = new int[numNodes];
		SortedSet unsettledNodes = new TreeSet(new shortestDistanceComparator());
		Set settledNodes = new HashSet();
		
		for(int i = 0; i < numNodes; i++)
		{
			mDistances[i] = 100000; //infinity : )
			mPrevious[i] = -1;
		
		}	
		
		mDistances[mStartNode] = 0;
		unsettledNodes.add(new Integer(mStartNode));
		
		while(!unsettledNodes.isEmpty())
		{
			int min = extractMin(unsettledNodes);
			if(min == -1)
				System.out.println("oops min is -1");
			settledNodes.add(new Integer(min));
			relaxNeighbors(min,settledNodes,unsettledNodes);
		}	
		dumpToSolutionTable();
		//printPrevious();	
		//printSolutionTable();
	}
	void relaxNeighbors(int node, Set settledNodes, SortedSet unsettledNodes)
	{
		//as good a rotation as any. these are the theoretical offsets to find
		//the neighbors of node
		int xdeltas[] = {1,0,-1,0};
		int ydeltas[] = {0,1,0,-1};
		int x =  getXfromNode(node);
		int y = getYfromNode(node);
		
		for(int i = 0; i < 4; i++)
		{
			int nx = x+xdeltas[i];
			int ny = y+ydeltas[i];
			int neighbor = ny*mWidth+nx;
			//test for out of bounds
			if(	nx < 0 || nx >= mWidth ||
				ny < 0 || ny >= mHeight)
				continue;
			//test whether its already been settled
			if(settledNodes.contains(new Integer(neighbor)))
			{
				//System.out.println(""+neighbor+" already in settled - node is"+node);
				continue;
			}
			//test whether it is upassable
			int cost = getCost(mTiles[nx][ny]);
			if(cost == -1)
			{
				//mSolutionTable[node][neighbor] = -1;
				//System.out.println(""+neighbor+" is equal to -1");
				continue;
			}	
			if( mDistances[neighbor] > mDistances[node] + cost)   // a shorter distance exists
			{    
				mDistances[neighbor] = mDistances[node] + cost;
				mPrevious[neighbor] = node;
				unsettledNodes.add(new Integer(neighbor));
			} 
		} 
	}
	void dumpToSolutionTable()
	{
		int numNodes = mWidth*mHeight;
		for(int i = 0; i < numNodes; i++)
		{
			mSolutionTable[i][i].value=(short)(i);
			int start = i;
			Vector v = new Vector();
			while(mPrevious[start] != -1)
			{
				mSolutionTable[i][mPrevious[start]].value=(short)(start);
				start = mPrevious[start];
			}
		}
	
	}
	void printSolutionTable()
	{
		int numNodes = mWidth*mHeight;
		for(int j = -1; j < numNodes; j++)
		{
			for(int i = -1; i < numNodes; i++)
			{
				if(j == -1)
				{
					if(i == -1)
						System.out.print("");
					else
						System.out.print("\t"+i);
				}
				else
				{
					if(i == -1)
						System.out.print(""+j);
					else
					{
						System.out.print("\tv"+mSolutionTable[i][j].value);
						System.out.print(" c"+mSolutionTable[i][j].cost);
					}
				}
			
			
			}
			System.out.println("");
		}
	
	}
	void printPrevious()
	{
		System.out.println("previous table------------------");
		for(int i = 0; i < mWidth*mHeight; i++)
			System.out.print("\t"+mPrevious[i]);
		System.out.println("--------------------------------");
	}
	int getXfromNode(int node)
	{
		return ( node%mWidth);
	}	
	int getYfromNode(int node)
	{
		return (node/mWidth);
	}
	int getNodeFromXY(int x, int y)
	{
		return (y * mWidth+x);
	}
	//the nodes must be sorted for this to work
	int extractMin(SortedSet nodes)
	{
	    Integer I = (Integer) nodes.first();
	    nodes.remove(I);
	    return I.intValue();
	}
	int getCost(byte val)
	{
		int cost = 1;
		switch(val)
		{
/*			case TERRAIN_WATER:
				cost = 2;
				break;
			case TERRAIN_SPACE:
				cost = 1;
				break;
			case TERRAIN_NORMAL:
				cost = 1;
				break;
			case TERRAIN_SLOW:
				cost = 2;
				break;
			case TERRAIN_SLOWER:
				cost = 3;
				break;
*/			case TERRAIN_NOMOVE:
				cost = -1;
				break;
/*			case TERRAIN_FIREDMG:
				cost = 2;
				break;
			case TERRAIN_SWAMPDMG:
				cost = 2;
				break;*/
			default:
				cost = 1;
				break;
				
		}
		return cost;
	}

}