/** 
 * smCanvas.java
 *
 * Description:	
 * @author			Nikita Mikros (c)2003 Black Hammer Game
 * @version			
 
 Draws the grid of the map and stores the data about the grid
 */
 
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;

public class smCanvas extends Canvas
{
	final short LIGHT = 0;
	final short SWAMP = 1;
	final short FIRE = 2;
	final short ICE = 3;
	
	final short mMaxWidth = 64;
	final short mMaxHeight = 64;
	final short NUM_TYPES = 24;
	public final short SPACE_UNIT_INDEX_OFFSET = 23;
	public final short DECOY_TYPE_INDEX_OFFSET = 24;


	public final byte CONQUEST = 0;
	public final byte RESCUE_HERO = 1;
	public final byte PROTECT_HOMEWORLD = 2;
	public final byte KING_OF_THE_STATION = 3;
	public final byte CAPTURE_HOMEWORLD = 4;
	public final byte BODY_COUNT = 5;
	public final byte GROUND_BATTLE = 6;
		
	public final byte SINGLE_PLAYER = 1;
	public final byte MULTI_PLAYER = 2;
	
	public boolean mShowGrid = true;
	public int mGridTileSize = 12;
	public int mSelectedX = 0;
	public int mSelectedY = 0;
	public int mSelectedIndex[] = new int[1];
	
	public short[][][] mType = new short[12][mMaxWidth][mMaxHeight];
	public short[][][] mSpecific = new short[12][mMaxWidth][mMaxHeight];
	public short[][][] mOwner = new short[12][mMaxWidth][mMaxHeight];
	public short[][][] mPlayerResources = new short[12][2][5];
	public String[][] mPlanetNames = new String[mMaxWidth][mMaxHeight];
	public String[][] mPlanetFileNames = new String[mMaxWidth][mMaxHeight];
	public Vector[][][] mSubUnits = new Vector[12][mMaxWidth][mMaxHeight];
	public short[][][] mIsSpecial = new short[12][mMaxWidth][mMaxHeight];//0 regular unit 1 special
	
	public short mClipType;
	public short mClipSpecific;
	public short mClipOwner;
	public short mClipIsSpecial;
	public String mClipPlanetName;
	public String mClipFileName;
	public Vector mClipSubUnits;
	public byte mGameType = CONQUEST;
	public byte mNumPlayers = SINGLE_PLAYER;
	public short mNumTurns = -1;
	public short mTimePerTurn = -1;
		
	Color mPlayerColors[] = new Color[2];

	String mDecoyStrings[] = {
		"LIGHT_SHIP",			//23
		"SWAMP_SHIP",			//24
		"FIRE_SHIP",			//25
		"ICE_SHIP",				//26
		"LIGHT_SHIP2",			//27
		"SWAMP_SHIP2",			//28
		"FIRE_SHIP2",			//29
		"ICE_SHIP2",			//30
		"LIGHT_MOTHER_SHIP",	//31
		"SWAMP_MOTHER_SHIP",	//32
		"FIRE_MOTHER_SHIP",		//33
		"ICE_MOTHER_SHIP",		//34
		"BATTLE_CRUISER",		//35
		"CARGO_SHIP_S",			//36
		"CARGO_SHIP_M",			//37
		"CARGO_SHIP_L",			//38
		"SCIENCE_SHIP",			//39
	};
	
	String mTypeStrings[] = {
		"NONE",
		"LIGHT_SHIP",			//23
		"SWAMP_SHIP",			//24
		"FIRE_SHIP",			//25
		"ICE_SHIP",				//26
		"LIGHT_SHIP2",			//27
		"SWAMP_SHIP2",			//28
		"FIRE_SHIP2",			//29
		"ICE_SHIP2",			//30
		"LIGHT_MOTHER_SHIP",	//31
		"SWAMP_MOTHER_SHIP",	//32
		"FIRE_MOTHER_SHIP",		//33
		"ICE_MOTHER_SHIP",		//34
		"BATTLE_CRUISER",		//35
		"CARGO_SHIP_S",			//36
		"CARGO_SHIP_M",			//37
		"CARGO_SHIP_L",			//38
		"SCIENCE_SHIP",			//39
		"DECOY",				//40
		"PIRATE",				//41
		"LIGHT_PLANET",			//42
		"SWAMP_PLANET",			//43
		"FIRE_PLANET",			//44
		"ICE_PLANET",			//45
		"STATION",				//46
	};

	short[] raceArray = {
			0,	
		SWAMP,//"UNDEAD",					//1
		-1,	//"RANGED",					    //2
		-1,	//"GRUNT",					    //3
		-1,	//"MORTAR_MAN",				    //4
		-1,	//MORTAR_SIEGE				    //5
		-1,	//"LARGE_TANK",				    //6
		-1,	//"SIEGE_TANK",				    //7
		-1,	//"LIGHT_MECH",				    //8
		SWAMP,	//"SWAMP_MECH",			    //9
		FIRE,	//"FIRE_MECH",			    //10
		ICE,	//"ICE_MECH",			    //11
		LIGHT,	//"LIGHT_SPECIAL",		    //12
		SWAMP,	//"SWAMP_SPECIAL",		    //13
		FIRE,	//"FIRE_SPECIAL",		    //14
		ICE,	//"ICE_SPECIAL",		    //15
		LIGHT,	//"LIGHT_FORCEFIELD",	    //16
		SWAMP,	//"SWAMP_FORCEFIELD",	    //17
		FIRE,	//"FIRE_FORCEFIELD",	    //18
		ICE,	//"ICE_FORCEFIELD",		    //19
		LIGHT,	//"LIGHT_FORCEFIELD_A",	    //20
		SWAMP,	//"SWAMP_FORCEFIELD_A",	    //21
		FIRE,	//"FIRE_FORCEFIELD_A",	    //22
		ICE,	//"ICE_FORCEFIELD_A",	    //23
		LIGHT,	//"LIGHT_SHIP",			    //24
		SWAMP,	//"SWAMP_SHIP",			    //25
		FIRE,	//"FIRE_SHIP",			    //26
		ICE,	//"ICE_SHIP",			    //27
		LIGHT,	//"LIGHT_SHIP2",		    //28
		SWAMP,	//"SWAMP_SHIP2",		    //29
		FIRE,	//"FIRE_SHIP2",			    //30
		ICE,	//"ICE_SHIP2",			    //31
		LIGHT,	//"LIGHT_MOTHER_SHIP",	    //32
		SWAMP,	//"SWAMP_MOTHER_SHIP",	    //33
		FIRE,	//"FIRE_MOTHER_SHIP",	    //34
		ICE,	//"ICE_MOTHER_SHIP",	    //35
		-1,	//"BATTLE_CRUISER",			    //36
		-1,	//"CARGO_SHIP_S",			    //37
		-1,	//"CARGO_SHIP_M",			    //38
		-1,	//"CARGO_SHIP_L",			    //39
		-1,	//"SCIENCE_SHIP",			    //40
		0,	//DECOY						    //41
		0,	//PIRATE					    //42
		LIGHT,	//"FIRE_PLANET",			//43
		SWAMP,	//"ICE_PLANET",				//44
		FIRE,	//"LIGHT_PLANET",			//45
		ICE,	//"SWAMP_PLANET",			//46
		-1,	//"STATION",				    //47
		0,	//"TREE",					    //48
		0	//"STONE"						//49
		};				
	
			
	Image mTypeImages[] = new Image[NUM_TYPES];
	Image mExportImage = null;
	
	public boolean save(RandomAccessFile out) throws Exception
	{
		long startOffsets = 0;
		
		//3 bytes magic
		out.write('s');
		out.write('s');
		out.write('f');
		
		
		//1 byte version
		out.write(5);
		
		out.write(mGameType);//game type
		out.write(mNumPlayers);//single player/multiplayer
		out.write(mNumTurns);//number of turns
		out.write(0);//reserved
		
		out.writeShort(bhgUtils.swapBytes(mNumTurns));//num turns >= v5
		out.writeShort(bhgUtils.swapBytes(mTimePerTurn));	//
		
		
		//write out planet files
		TreeMap tm = new TreeMap();
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				short unittype = (short)(mType[0][i][j]);
				if(this.isPlanet(unittype))
				{
					if(tm.get(this.mPlanetFileNames[i][j]) == null)
					{
						//if its not in the tree prepare the content
						byte[] tempString = toBytes(mPlanetFileNames[i][j],64);
						tm.put(mPlanetFileNames[i][j],tempString);		
					
					}
				}
				
			}
		}
		//number of planets
		int numPlanets = tm.size();
		out.writeInt(bhgUtils.swapBytes(numPlanets));
		//now write out the actual elements
		Vector v = new Vector(tm.values());
		Enumeration enum1 = v.elements();
		while(enum1.hasMoreElements())
		{
			byte[] tempString = (byte[])enum1.nextElement();
			out.write(tempString);
		}
		
		
		//for each scenario write out the following info
		//player resources
		//units on board including ownership
		//sub unit types and hp
		
		//player resources
		for(int k = 0; k < 12; k++)
		{
			for(int j = 0; j < 2; j++)
			{
				for(int i = 0; i < 5; i++)
				{
					short val = bhgUtils.swapBytes(mPlayerResources[k][j][i]);
					out.writeShort(val);
				}
			}
		}
		//put in placeholder values for the offsets
		startOffsets = out.getFilePointer();//save the beginning of the offsets
		for(int k = 0; k < 12; k++)
			out.writeInt(0);
		for(int k = 0; k < 12; k++)
		{
			long numObjectPosition = out.getFilePointer();
			//now go back and fill in the offset marker
			out.seek(startOffsets+k*4);//go back
			int objmarker = (int)numObjectPosition;//make it into an int for java : (
			out.writeInt(bhgUtils.swapBytes(objmarker));//write out the value
			out.seek(numObjectPosition);//put us back to where we were
			
			out.writeInt(0);
			long endPosition = numObjectPosition;
			int numObjects = 0;
			//System.out.println("combination: "+k);
			for(int j = 0; j < mMaxHeight; j++)
			{
				for(int i = 0; i < mMaxWidth; i++)
				{
					if(mType[k][i][j] > 21)
					{
						short unittype = (short)(mType[k][i][j]);
						//System.out.println("\tabout to save out unit of type:"+unittype);
						out.writeShort(bhgUtils.swapBytes((short)i));//x
						out.writeShort(bhgUtils.swapBytes((short)j));//y
						
						out.writeShort(bhgUtils.swapBytes(unittype));//type
						out.writeShort(bhgUtils.swapBytes((short)(100)));//HP
						out.writeShort(bhgUtils.swapBytes((short)mIsSpecial[k][i][j]));//special? >= v.5
						out.writeShort(bhgUtils.swapBytes(this.mOwner[k][i][j]));//owner
						short race = raceArray[unittype];
						if(race == -1)
						{
							if(
								this.isCargoShip(mType[k][i][j]) || 
								this.isScienceShip(mType[k][i][j]) ||
								this.isBattleCruiser(mType[k][i][j]) 
							)
							{
								if(mOwner[k][i][j] != 2)
								{
									race =(short) this.getPlayerRace(mOwner[k][i][j]);
								}
								else 
									race = 0;
							}
							else
								race = 0;
						
						}
						out.writeShort(bhgUtils.swapBytes(race));
						short specifics = this.mSpecific[k][i][j];
						if(specifics == -1)
							specifics = 0;
						out.writeShort(bhgUtils.swapBytes(specifics));//how many factories or which upgrade
						
						if(isPlanet(unittype))
						{
							//write out planet filenames
							byte[] tempString = toBytes(mPlanetFileNames[i][j],64);
							out.write(tempString);
							//write out planet name
							tempString = toBytes(mPlanetNames[i][j],64);
							out.write(tempString);
						}
						//System.out.println("");
						if(mSubUnits[k][i][j] != null)
						{
							short num_subs = (short)(mSubUnits[k][i][j].size());
							
							//if the unit is not owned by anyone do not record the sub units
							if(mOwner[k][i][j] == 2)
								num_subs = 0;
							//if the unit is a freighter ignore any subs
							if(this.isCargoShip(mType[k][i][j]))
								num_subs = 0;
							out.writeShort(bhgUtils.swapBytes(num_subs));//num subunits
							if(num_subs > 0)
							{
								System.out.println("\tsub units: "+num_subs);
								Enumeration enum = mSubUnits[k][i][j].elements();
								while(enum.hasMoreElements())
								{
									subUnit su = (subUnit)enum.nextElement();
									short temp = 0;//was -1
									short subrace = raceArray[su.id];
									if(subrace == -1)
										subrace = race;
									out.writeShort(bhgUtils.swapBytes(temp));//x
									out.writeShort(bhgUtils.swapBytes(temp));//y
									out.writeShort(bhgUtils.swapBytes((short)su.id));//type
									out.writeShort(bhgUtils.swapBytes((short)su.hp));//HP
									out.writeShort(bhgUtils.swapBytes((short)su.isSpecial));//special?  >= v5
									out.writeShort(bhgUtils.swapBytes(mOwner[k][i][j]));//owner
									System.out.println("\t\ttype: "+su.id);
									
									out.writeShort(bhgUtils.swapBytes(subrace));//race
									out.writeShort(bhgUtils.swapBytes((short)(0)));//editor subunits cannot have subunits
								}
							}
						}
						else
							out.writeShort(bhgUtils.swapBytes((short)(0)));
						
						endPosition = out.getFilePointer();
						numObjects++;
					}
				}
			}
			if(numObjects > 0)
			{
				out.seek(numObjectPosition);
				out.writeInt(bhgUtils.swapBytes(numObjects));
				out.seek(endPosition);
			}
		}
		
		return true;
	}
	public byte[] toBytes(String s, int maxSize)
	{
		byte[] b2 = new byte[maxSize];
		int end = 0;
		if(s != null)
		{
			byte[] b1 = s.getBytes();
			int theSize = b1.length > (maxSize-1) ? (maxSize-1) : b1.length;
			end = theSize;
			for(int i = 0; i < theSize; i++)
			{
				b2[i] = b1[i];
			
			}
		}
		//make sure this guy is null terminated
		b2[end] = 0;
		return b2;
	
	}
	public boolean read(DataInputStream in) throws Exception
	{
		byte version;
		byte[] magic = new byte[3];
		in.read(magic,0,3);

		if(magic[0] != 's' || magic[1] != 's' || magic[2] != 'f')
			return false;
		version = (byte)in.read();

		if(version >= 3)
		{
			mGameType = (byte)in.read();//game type
			mNumPlayers = (byte)in.read();//single player/multiplayer
			mNumTurns = (byte)in.read();//number of turns (obsolete!!!!)
			in.read();//reserved
		}
		
		if(version >= 5)
		{
			mNumTurns = bhgUtils.swapBytes(in.readShort());
			mTimePerTurn = bhgUtils.swapBytes(in.readShort());
		
			int numPlanets = bhgUtils.swapBytes(in.readInt());
			
			byte[] tempBytes = new byte[64];
			// planet filenames
			for(int i = 0; i < numPlanets; i++)
			{
				in.read(tempBytes,0,64);
				System.out.println( new String(tempBytes));
			}
		
		}
		//player resources
		for(int k = 0; k < 12; k++)
		{
			for(int j = 0; j < 2; j++)
			{
				for(int i = 0; i < 5; i++)
				{
					short raw = in.readShort();
					short val = bhgUtils.swapBytes(raw);
					mPlayerResources[k][j][i] = val;
				}
			}
		}
		//since we want to load all the scenarios we can skip over the indices telling
		//us where a particular scenario is	
		in.skip(48);
		
		for(int k = 0; k < 12; k++)
		{
			int numObjects = bhgUtils.swapBytes(in.readInt());
			for(int j = 0; j < numObjects; j++)
			{
				short x = bhgUtils.swapBytes(in.readShort());//x			
				short y = bhgUtils.swapBytes(in.readShort());//y
				//System.out.println("x = "+x+"y = "+y);		
				
				short type = bhgUtils.swapBytes(in.readShort());//type
				if(version < 2 && type > 38)
					type += 2;
				if(version < 3 && type >=4)
					type++; 
				if(version < 4)
					type++;
				mType[k][x][y] = type;
				//mType[k][x][y] -= SPACE_UNIT_INDEX_OFFSET;
				
				in.readShort();//HP
				
				//is special?
				short special;
				if(version >= 5)
					special = bhgUtils.swapBytes(in.readShort());
				else
					special = 0;
				this.mIsSpecial[k][x][y] = special;
				
				mOwner[k][x][y] = bhgUtils.swapBytes(in.readShort());//owner
				in.readShort();//race
				mSpecific[k][x][y] = bhgUtils.swapBytes(in.readShort());//factories or upgrade info
				
				if(isPlanet(mType[k][x][y]))
				{
					byte[] tempBytes = new byte[64];
					// planet filenames
					in.read(tempBytes,0,64);
					mPlanetFileNames[x][y] = new String(tempBytes);
					
					//planet names
					in.read(tempBytes,0,64);
					mPlanetNames[x][y] = new String(tempBytes);					
				}
				
				int subunits = bhgUtils.swapBytes(in.readShort());//num sub units 
				if(subunits > 0)
				{
					mSubUnits[k][x][y]= new Vector();
					for(int su_i = 0; su_i < subunits; su_i++)
					{
						subUnit su = new subUnit();
						in.readShort();//x
						in.readShort();//y
						su.id = bhgUtils.swapBytes(in.readShort());
						if(version < 3 && su.id >=4)
							su.id++; 
						if(version < 4)
							su.id++;

						su.hp = bhgUtils.swapBytes(in.readShort());
						
						//is the unit "special"?
						if(version >= 5)
							su.isSpecial =  bhgUtils.swapBytes(in.readShort());
						else
							su.isSpecial = 0;
						in.readShort();//owner
						in.readShort();//race
						in.readShort();	//subunits - note 
										//if a unit has subunits it cannot be read
										//byt the editor although the game should be
										//able to read it - a case of this would be the
										//battlecruiser which can hold other ships
						mSubUnits[k][x][y].add(su);
					}
				}
			}
		}					
		return true;
	}

	
	//constructor
	public smCanvas()
	{
		clear();


		Toolkit t = java.awt.Toolkit.getDefaultToolkit();
//		for(int i = 0; i < NUM_TERRAIN_TYPES; i++)
		mTypeImages[0] = t.getImage("img/lightship1.gif");
		mTypeImages[1] = t.getImage("img/swampship1.gif");
		mTypeImages[2] = t.getImage("img/fireship1.gif");
		mTypeImages[3] = t.getImage("img/iceship1.gif");
		mTypeImages[4] = t.getImage("img/lightship2.gif");
		mTypeImages[5] = t.getImage("img/swampship2.gif");
		mTypeImages[6] = t.getImage("img/fireship2.gif");
		mTypeImages[7] = t.getImage("img/iceship2.gif");
		mTypeImages[8] = t.getImage("img/lightmother.gif");
		mTypeImages[9] = t.getImage("img/swampmother.gif");
		mTypeImages[10] = t.getImage("img/firemother.gif");
		mTypeImages[11] = t.getImage("img/icemother.gif");
		mTypeImages[12] = t.getImage("img/battlecruiser.gif");
		mTypeImages[13] = t.getImage("img/cargo1.gif");
		mTypeImages[14] = t.getImage("img/cargo2.gif");
		mTypeImages[15] = t.getImage("img/cargo3.gif");
		mTypeImages[16] = t.getImage("img/sciencevessel.gif");
		mTypeImages[17] = t.getImage("img/decoy.gif");
		mTypeImages[18] = t.getImage("img/pirate.gif");
		mTypeImages[19] = t.getImage("img/lightplanet.gif");
		mTypeImages[20] = t.getImage("img/swampplanet.gif");
		mTypeImages[21] = t.getImage("img/fireplanet.gif");
		mTypeImages[22] = t.getImage("img/iceplanet.gif");
		mTypeImages[23] = t.getImage("img/upgradestation.gif");
		
		for(int i = 0; i < NUM_TYPES; i++)
			t.prepareImage(mTypeImages[i],-1,-1,null);
		
		
	}
	
	public void clear()
	{
		mGameType = this.CONQUEST;
		mNumPlayers = this.SINGLE_PLAYER;
		mNumTurns = -1;
		mTimePerTurn = -1;
		
		//zero out the map
		for(int k = 0; k < 12; k++)
		{
			for(int j = 0; j < mMaxHeight; j++)
			{
				for(int i = 0; i < mMaxWidth; i++)
				{
					mType[k][i][j] = 0;
					mSpecific[k][i][j] = 0;
					mOwner[k][i][j] = 2;
					mSubUnits[k][i][j] = null;
					if(k == 0)
					{
						mPlanetNames[i][j] = new String("");
						mPlanetFileNames[i][j] = new String("");
					}
				}
			}
		}
		//reset the player's resources
		for(int k = 0; k < 12; k++)
		{
			for(int j = 0; j < 2; j++)
			{
				for(int i = 0; i < 5; i++)
				{
					if(i == 0)
						mPlayerResources[k][j][i] = 200;
					else
						mPlayerResources[k][j][i] = 100;
				}
			}
		}
	}
	public void update(Graphics g)
	{
		paint(g);
	}
	public boolean isPlanet(int type)
	{
		return (type >= (43) && type <= (46));
	}
	public boolean isStation(int type)
	{
		return (type == (47));
	}
	public boolean isCargoShip(int type)
	{
		return (type >= (37) && type <= (39));	
	}
	public boolean isDecoy(int type)
	{
		return (type == 41);
	}
	public boolean isScienceShip(int type)
	{
		return (type == (40));	
	}
	public boolean isBattleCruiser(int type)
	{
		return (type == (36));	
	}
	public boolean isPirate(int type)
	{
		return (type == (42));
	}
	public int getPlayerRace(int whichPlayer)
	{
		int index = (mSelectedIndex[0]/4)+((mSelectedIndex[0]%4 > 0)?1:0);
		int x = index%4;
		int y = index/4;
		
		return (whichPlayer == 0?x:y);
	
	}
	public void setTile(int x, int y, short CO, short type, short specifics, String filename, String planetname, short isSpecial)
	{
		if(x >= 0 && x < mMaxWidth && y >= 0 && y < mMaxHeight)
		{
			//if its a planet or will be a planet then set all layers
			if(	isPlanet(type)||
				isPlanet(mType[mSelectedIndex[0]][x][y])||
				isStation(type) ||
				isStation(mType[mSelectedIndex[0]][x][y])
			)
			{
				if(isPlanet(type) || isPlanet(mType[mSelectedIndex[0]][x][y]))
				{
					mPlanetFileNames[x][y] = new String(filename);
					mPlanetNames[x][y] = new String(planetname);
				}
				for(int i = 0; i < 12; i++)
				{
					mType[i][x][y] = type;
					
					if(type == 0)
					{
						mOwner[i][x][y] = CO;
						mSpecific[i][x][y] = specifics;
					}
				}
			
			}	
			for(int i = 0; i < mSelectedIndex.length; i++)
			{
			
				mType[mSelectedIndex[i]][x][y] = type;
				mOwner[mSelectedIndex[i]][x][y] = CO;
				mSpecific[mSelectedIndex[i]][x][y] = specifics;
				mIsSpecial[mSelectedIndex[i]][x][y] = isSpecial;
			}
			
			
			//now draw it
			paintTile(getGraphics(),x,y);
		}
	}
	public void selectTile(int x, int y)
	{
		paintTile(getGraphics(),mSelectedX,mSelectedY);
		mSelectedX = x;
		mSelectedY = y;
		paintSelected(getGraphics());
	}
	
	public boolean isDoubleBuffered()
	{
		return true;
	}
	public void cut()
	{
		int x = mSelectedX;
		int y = mSelectedY;
		mClipType = mType[mSelectedIndex[0]][x][y];
		mClipSpecific = mSpecific[mSelectedIndex[0]][x][y];
		mClipIsSpecial = mIsSpecial[mSelectedIndex[0]][x][y];
		mClipOwner = mOwner[mSelectedIndex[0]][x][y];;
		mClipPlanetName = new String(mPlanetNames[x][y]) ;
		mClipFileName = new String(mPlanetFileNames[x][y]);
		if(mSubUnits[mSelectedIndex[0]][x][y] != null && mSubUnits[mSelectedIndex[0]][x][y].size() != 0)
			mClipSubUnits = new Vector(this.mSubUnits[mSelectedIndex[0]][x][y]);
		else
			mClipSubUnits = null;	
			
		setTile(x,y,(short)2,(short)0,(short)0,"","",(short)0);
	
	}
	public void copy()
	{
		int x = mSelectedX;
		int y = mSelectedY;
		mClipType = mType[mSelectedIndex[0]][x][y];
		mClipSpecific = mSpecific[mSelectedIndex[0]][x][y];
		mClipIsSpecial = mIsSpecial[mSelectedIndex[0]][x][y];
		mClipOwner = mOwner[mSelectedIndex[0]][x][y];;
		mClipPlanetName = new String(mPlanetNames[x][y]) ;
		mClipFileName = new String(mPlanetFileNames[x][y]);
		if(mSubUnits[mSelectedIndex[0]][x][y] != null && mSubUnits[mSelectedIndex[0]][x][y].size() != 0)
			mClipSubUnits = new Vector(this.mSubUnits[mSelectedIndex[0]][x][y]);
		else
			mClipSubUnits = null;
	
	}
	public void paste()
	{
		int x = mSelectedX;
		int y = mSelectedY;
		setTile(x,y,mClipOwner,mClipType,mClipSpecific,mClipFileName,mClipPlanetName,mClipIsSpecial);
		if(mClipSubUnits != null)
		{
			for(int i = 0; i < mSelectedIndex.length; i++)
				mSubUnits[mSelectedIndex[i]][x][y] = new Vector(mClipSubUnits);
		}
	}
	public void paintTile(Graphics g, int x, int y)
	{
		g.setColor(Color.gray);
		g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
		if(mType[mSelectedIndex[0]][x][y] != 0)
		{	
			if(mOwner[mSelectedIndex[0]][x][y] != 2)
			{
				g.setColor(mPlayerColors[mOwner[mSelectedIndex[0]][x][y]]);
				g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
			}
			g.drawImage(mTypeImages[mType[mSelectedIndex[0]][x][y]-1-SPACE_UNIT_INDEX_OFFSET],x*mGridTileSize,y*mGridTileSize,null);
		}
		else
		{
			g.setColor(Color.darkGray);
			g.drawRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
		}
	}
	public void paintSelected(Graphics g)
	{
		g.setColor(Color.red);
		g.drawRect(mSelectedX*mGridTileSize,mSelectedY*mGridTileSize,mGridTileSize-1,mGridTileSize-1);
	}
	public void paint(Graphics g)
	{
		g.setColor(Color.gray);
		g.fillRect(0,0,mMaxWidth*mGridTileSize,mMaxHeight*mGridTileSize);
		
		//draw grid
		g.setColor(Color.darkGray);
		for(int j = 0; j <= mMaxHeight*mGridTileSize; j+= mGridTileSize)
			g.drawLine(0,j,mMaxWidth*mGridTileSize,j);
		for(int i = 0; i <= mMaxWidth*mGridTileSize; i+= mGridTileSize)
			g.drawLine(i,0,i,mMaxHeight*mGridTileSize);
		//draw tiles
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				if(mType[mSelectedIndex[0]][i][j] - this.SPACE_UNIT_INDEX_OFFSET != 0)
					paintTile(g,i,j);
			}
		}
		//draw selection
		paintSelected(g);
	}	
	public void paintExport(Graphics g)
	{
		g.setColor(Color.black);
		g.fillRect(0,0,mMaxWidth*mGridTileSize,mMaxHeight*mGridTileSize);
		
		//draw tiles
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				if(mType[mSelectedIndex[0]][i][j] - this.SPACE_UNIT_INDEX_OFFSET != 0)
					paintExportTile(g,i,j);
			}
		}
	}
	public void paintExportTile(Graphics g, int x, int y)
	{
		g.setColor(Color.black);
		g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
		if(mType[mSelectedIndex[0]][x][y] != 0)
		{	
			if(mOwner[mSelectedIndex[0]][x][y] != 2)
				g.setColor(mPlayerColors[mOwner[mSelectedIndex[0]][x][y]]);
			else if(this.isStation(mType[mSelectedIndex[0]][x][y]))
				g.setColor(Color.darkGray);
			else
				g.setColor(Color.lightGray);
				
			if(this.isPlanet(mType[mSelectedIndex[0]][x][y]))
				g.fillRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
			else if(!this.isPirate(mType[mSelectedIndex[0]][x][y]))
				g.fillRect(x*mGridTileSize+2,y*mGridTileSize+2,mGridTileSize-4,mGridTileSize-4);
				

			//g.drawImage(mTypeImages[mType[mSelectedIndex[0]][x][y]-1-SPACE_UNIT_INDEX_OFFSET],x*mGridTileSize,y*mGridTileSize,null);
		}
		/*else
		{
			g.setColor(Color.darkGray);
			g.drawRect(x*mGridTileSize,y*mGridTileSize,mGridTileSize,mGridTileSize);
		}	*/	
	
	}
	public void exportBMP(DataOutputStream out) throws Exception
	{
		int tile_size = 12;//type==EXPORT_NORMAL?8:16;
		//bit o magic
		out.writeByte('B');
		out.writeByte('M');
		
		//complet file size
		out.writeInt(bhgUtils.swapBytes((int)(0x36 + mMaxWidth*mMaxHeight*64*4)));
		
		//reserved
		out.writeInt(0);
		
		//offset to the beginning of the bitmap data 
		out.writeInt(bhgUtils.swapBytes((int)0x36));
		
		//header size
		out.writeInt(bhgUtils.swapBytes((int)0x28));
		//width
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*tile_size));
		//height
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*tile_size));
		
		//planes
		out.writeShort(bhgUtils.swapBytes((short)1));
		//bits per pixel
		out.writeShort(bhgUtils.swapBytes((short)32));
		
		//compression (none)
		out.writeInt(bhgUtils.swapBytes((int)0));
		
		//size of the bitmap (in bytes)
		out.writeInt(bhgUtils.swapBytes(mMaxWidth*mMaxWidth *tile_size*tile_size*4));
		
		//hor  & vert rez
		out.writeInt(0);
		out.writeInt(0);
		
		//colors
		out.writeInt(0);
		//important colors
		out.writeInt(0);
		
		int x,y;
		
		int[] pixels = new int[mMaxWidth * mMaxHeight*tile_size*tile_size];
		int pixWidth = mMaxWidth*tile_size;
		int pixHeight = mMaxHeight*tile_size;
		//Image im =  createImage(pixWidth, pixHeight);
		if(mExportImage == null)
		{
			mExportImage =  createImage(mMaxWidth*tile_size, mMaxHeight*tile_size);

		}
			//this.primeImages();
		Image im = mExportImage;
		Graphics ig = im.getGraphics();

		/*if(type == EXPORT_TEXTURE)
			paintTerrain(ig);
		else if(type == EXPORT_HEIGHT)
			paintHeightMap(ig);
		else*/
			paintExport(ig);
		
		
		PixelGrabber pg = new PixelGrabber(im, 0, 0, pixWidth, pixHeight, pixels, 0, pixWidth);
		try 
		{
	    	pg.grabPixels();
		} 
		catch (InterruptedException e) 
		{
	    	System.err.println("interrupted waiting for pixels!");
	    }

		
		for(int j = 0; j < pixHeight; j++)
		{
			for(int i = 0; i < pixWidth;i++)
			{
				x = i;
				y = pixHeight - (j+1);
				/*if	(y >= (mOffsetY * 8) && y < ((mOffsetY + mHeight) * 8) && 
					(x >= (mOffsetX * 8) && x < ((mOffsetX + mWidth) * 8)))
					{*/
						int pixX = i;// - (mOffsetX*8);
						int pixY = y;//- (mOffsetY*8);
						
						//System.out.println("pixX"+pixX+"pixY"+pixY+"pixWidth"+pixWidth);
						//System.out.println("j"+j+"mOffsetY*8"+mOffsetY*8+"y"+y);
						out.writeInt(bhgUtils.swapBytes(pixels[pixY * pixWidth + pixX]));
					
				/*	}
					
				else
					out.writeInt(0xffffff00);*/
			}
		}
		
	
	}

}