/** 
 * smPickerCanvas.java
 *
 * Description:	
 * @author			Nikita Mikros (c)2003 Black Hammer Game
 * @version			
 
 Draws the grid of the map and stores the data about the grid
 */
 import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;

public class smPickerCanvas extends Canvas
{
	final short LIGHT = 0;
	final short SWAMP = 1;
	final short FIRE = 2;
	final short ICE = 3;
	
	final short mMaxWidth = 4;
	final short mMaxHeight = 4;
	
	public boolean mShowGrid = true;
	public int mGridTileSize = 16;
	public int mSelectedX = 0;
	public int mSelectedY = 0;
	
	int mSelected[] = new int[16];
	
	Color mCOColors[]=
	{
		new Color(255,247,187),//light
		new Color(146,39,143),//swamp
		new Color(158,11,14),//fire
		new Color(0,191,243),//ice
	};
		
	//constructor
	public smPickerCanvas()
	{
		mSelectedX = 1;
		mSelectedY = 0;
		for(int i = 0; i < 16; i++)
			mSelected[i] = 0;
		mSelected[mSelectedY*mMaxWidth + mSelectedX] = 1;
	}
	
	public void update(Graphics g)
	{
		paint(g);
	}

	public int selectTile(int x, int y)
	{
		//test whether its on the diagonal
		if(x == y)
			return -1;
		//test whether its the same one
		if(mSelectedX == x && mSelectedY == y)
			return -1;	
			
		mSelectedX = x;
		mSelectedY = y;


		//return the index
		int num = y*mMaxWidth + x;
		for(int i = 0; i < 16; i++)
			mSelected[i] = 0;
		mSelected[num] = 1;
		paint(getGraphics());
		return convertToIndex(num);
	}
	
	public int[] selectTileAdd(int x, int y)
	{
		//test whether its on the diagonal
		if(x == y)
			return null;
		if(x == mSelectedX && y == mSelectedY)
			return null;	

		//return the index
		int num = y*mMaxWidth + x;
		if(mSelected[num] == 2)
			mSelected[num] = 0;
		else if(mSelected[num] == 0)
			mSelected[num] = 2;
		
		Vector v = new Vector();
		for(int i = 0; i < 16; i++)
		{	
			if(mSelected[i] == 2)
			{
				num = convertToIndex(i);
				v.add(new Integer(num));
			}
		}
		if(v.size()>0)
		{
			int selected[] = new int[v.size() + 1];
			selected[0] = convertToIndex(mSelectedX,mSelectedY);
			for(int i = 1; i < selected.length; i++)
			{
				Integer I = (Integer)(v.get(i-1));
				selected[i] = I.intValue();
			}
			paint(getGraphics());
			return selected;
		}
		return null;
	}
	int convertToIndex(int x, int y)
	{
		int num = y*mMaxWidth + x;
		return convertToIndex(num);
	}
	int convertToIndex(int num)
	{
		int quotient =  (num/(mMaxWidth+1));
		num = num - (1 + quotient);
		return num;
	}
	public Color getPlayerColor(int whichPlayer)
	{
		int index = whichPlayer == 0?mSelectedX:mSelectedY;
		return (mCOColors[index]);
	
	}
	public boolean isDoubleBuffered()
	{
		return true;
	}
	public void paintTile(Graphics g, int x, int y)
	{
		int index = x + mMaxWidth*y;
			
		if(x == mSelectedX && y == mSelectedY)
		{
			g.setColor(Color.black);
		}
		else if(mSelected[index] == 2)
		{
			g.setColor(Color.gray);
		}
		else if(x == y)
		{
			g.setColor(Color.red);
		}
		else
		{
			g.setColor(Color.white);
		}
		g.fillRect((x+1)*mGridTileSize,(y+1)*mGridTileSize,mGridTileSize,mGridTileSize);
		g.setColor(Color.black);
		g.drawRect((x+1)*mGridTileSize,(y+1)*mGridTileSize,mGridTileSize,mGridTileSize);
	}
	public void paint(Graphics g)
	{
		g.setColor(Color.black);
		g.fillRect(0,0,(mMaxWidth+1)*mGridTileSize,(mMaxHeight+1)*mGridTileSize);
		for(int i = 0; i < 4; i++)
		{
			g.setColor(mCOColors[i]);
			g.fillRect(0,(i+1)*mGridTileSize,mGridTileSize,mGridTileSize);
			g.fillRect((i+1)*mGridTileSize,0,mGridTileSize,mGridTileSize);
		}
		//draw tiles
		for(int j = 0; j < mMaxHeight; j++)
		{
			for(int i = 0; i < mMaxWidth; i++)
			{
				paintTile(g,i,j);
			}
		}
	}	
	

}